#ifndef MP4V2_UTIL_OTHER_H
#define MP4V2_UTIL_OTHER_H

///////////////////////////////////////////////////////////////////////////////

namespace mp4v2 { namespace util {

///////////////////////////////////////////////////////////////////////////////

struct MP4V2_EXPORT FileSummaryInfo {
    typedef set<string>  BrandSet;

    // standard ftyp box attributes
    string   major_brand;
    uint32_t minor_version;
    BrandSet compatible_brands;

    uint32_t nlargesize;
    uint32_t nversion1;
    uint32_t nspecial;
};

/// Fetch mp4 file summary information.
///
/// This function fetches summary information for <b>file</b> and information
/// is stored in <b>info</b>.
///
/// @return On success <b>true</b>.
///     On failure <b>false</b>, and contents of <b>info</b> are undefined.
///
MP4V2_EXPORT
bool fileFetchSummaryInfo( MP4FileHandle file, FileSummaryInfo& info );

///////////////////////////////////////////////////////////////////////////////

/** Read an existing mp4 file and disable write-protection.
 *
 *  This function is identical to MP4Read() except that it disables
 *  write-protection for routines that would be incompatible with
 *  the MP4CloseCopy() workflow.
 *
 *  @param fileName pathname of the file to be read.
 *  @param verbosity bitmask of diagnostic details the library
 *      should print to stdout during its functioning.
 *
 *  @return On success a handle of the file for use in subsequent calls to
 *      the library.
 *      On error, #MP4_INVALID_FILE_HANDLE.
 *
 *  @see MP4SetVerbosity() for <b>verbosity</b> values.
 */
MP4V2_EXPORT
MP4FileHandle MP4ReadCopy( const string& fileName, uint32_t verbosity = 0 );

/// Copy-out and close an mp4 file.
/// This function writes out a copy of all the atoms from an open mp4
/// file. The new file is written out in optimized-form, so a subsequent
/// optimize operation is not necessary. Both files are then closed.
///
/// @param hFile handle of file to close.
/// @param copyFileName name of new file to copy to.
///
/// @return <b>true</b> on success, <b>false</b> on failure.
///
MP4V2_EXPORT
bool MP4CopyClose( MP4FileHandle hFile, const string& copyFileName );

///////////////////////////////////////////////////////////////////////////////

}} // namespace mp4v2::util

#endif // MP4V2_UTIL_OTHER_H
